// Fichier I2C_Slave.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Librairie pour la gestion de l'I2C en mode esclave   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 *                                                      		*
 ****************************************************************/



#include <i2c.h>	   		//gestion de la communication I2C
#include <p18f452.h>		// librairie du pic

#include "I2C_Slave.h"		// Header de la bibliothque
#include "led.h"

extern unsigned char NumSlotReel;						// Numro du slot courant

extern unsigned char Slot[5];							// Contenu du barillet
extern unsigned char FlagErreur;						// Pour signaler une erreur a l'asservissement
extern unsigned char FlagStocker;						// 1 = en train de stocker
extern unsigned char FlagBusy;							// 1 = en train d'executer un ordre

volatile unsigned char Buffer_Ordre[TAILLE_BUFFER]={0};	// Tableau qui bufferise les ordres  executer





// -----------------------------------------------------------------------
/* Initialise le module I2C
 * Configure les ports entre
 * Configure le module en esclave
 * Spcifie l'adresse et configure les interruptions
*/
void I2C_Init(void)
{
	TRIS_SCL = 1;				// Configure les ports en entre
	TRIS_SDA = 1;				// Car on est en mode esclave

	OpenI2C(SLAVE_7, SLEW_OFF);	// configuration du pic en esclave sur 7 bits d'adresse (+ une lecture criture), vitesse sur 100 Khz
 
	SSPADD = SLAVE_ADRESS;		// Adresse de l'esclave
	SSPCON1bits.CKP = 1;  		// On relache l'horloge

	PIE1bits.SSPIE=1;      		// autorisation de l'interruption de l'i2c 
  	PIR1bits.SSPIF=0; 			// on dsactive le bit SSPIF pour 
								// permettre a l'interruption d'avoir lieu
}




/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Interruptions                                //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////



/************************************Vecteur d'interruption Haut**********************************/

#pragma code InterruptVectorHigh = 0x08   //Priorit basse
void InterruptVectorHigh(void)  
{	
	_asm
		goto ServiceInterruptHigh	//Saut  la routine d'interruption haute
	_endasm
}
#pragma code



/************************************Interruption priorit Haute**********************************/
#pragma interrupt ServiceInterruptHigh
void ServiceInterruptHigh(void)
{
	static unsigned char Indice_Int=0;
	unsigned char i;
	if (PIR1bits.SSPIF) 									// Si interruption sur l'I2C
	{
		PIR1bits.SSPIF = 0; 								// On rautorise l'interruption		
		Buffer_Ordre[Indice_Int] = InterruptI2C();			// On appelle la routine I2C pour traiter la requete				
		if (Buffer_Ordre[Indice_Int])						// Si un nouvelle ordre est arriv ...
		{
			if (Buffer_Ordre[Indice_Int]==STOP_BARILLET)	// Si c'est un ordre d'arrt ...
			{
				for (i=0;i<TAILLE_BUFFER;i++)				// On vide intgralement le buffer
					Buffer_Ordre[i]=0;
			}
			else
				Indice_Int=(Indice_Int+1)%TAILLE_BUFFER;	// Sinon, on l'empile
		}
	}	

}
#pragma code





/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonctions		                               // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////


// -----------------------------------------------------------------------
/* Fonction  appeler lors d'un interruption sur l'I2C
*/
char InterruptI2C(void)
{
	unsigned char 			Adresse=0;				// Dernire adresse reue (Utilis pour dterminer s'il s'agit d'une transmission ou d'une rception)
	char 		 			ReceivedValue=0;		// Mmorise la dernire valeur reue
	
	unsigned char		 	Octet1 = 0;
	static unsigned char 	Octet2 = 0;

	// Transmission : le matre demande des donnes
	// Rception	: le matre envoie des donnes

	// -------------------------- Adresse / Transmission ----------------------------
	if(!SSPSTATbits.D_A && SSPSTATbits.R_W)		// S'il s'agit d'une adresse en mode transmission
	{
		Octet1 = Slot[NumSlotReel] << 6 | Slot[(NumSlotReel+1)%5] <<4 | Slot[(NumSlotReel+2)%5] <<2 | Slot[(NumSlotReel+3)%5];		
		SSPBUF = Octet1;						// On transmet la premire donne 
		SSPCON1bits.CKP = 1;  					// On relache l'horloge
		Octet2 = Slot[((NumSlotReel+4)%5)] << 6 | FlagStocker | FlagBusy ;
	}

	// -------------------------- Adresse / Rception ----------------------------
	if(!SSPSTATbits.D_A && !SSPSTATbits.R_W)	// S'il s'agit d'une adresse en mode rception
	{
		Adresse=SSPBUF;							// On lit la donne (l'adresse) pour vider le buffer		
	}
	
	// -------------------------- Donne / Transmission ----------------------------
	if(SSPSTATbits.D_A && SSPSTATbits.R_W)		// S'il s'agit d'une donne en mode transmission
	{
		SSPBUF = Octet2;						// On transmet la deuxime donne 
		SSPCON1bits.CKP = 1;  					// On relache l'horloge
	}
	
	// -------------------------- Donne / Rception ----------------------------
	if(SSPSTATbits.D_A && !SSPSTATbits.R_W)		// S'il s'agit d'une donne en mode rception
	{
		if (SSPSTATbits.BF)						// Vrifie qu'il y a une donne  lire 
		{
			ReceivedValue=SSPBUF;						// On lit la donne			
		}
	}		
	return ReceivedValue;	
}
